@@ -1,13 +1,13 @@
 package POE::Component::IKC::Responder;
 
 ############################################################
-# $Id: Responder.pm 311 2007-11-29 21:15:53Z fil $
+# $Id: Responder.pm 343 2009-03-20 07:51:40Z fil $
 # Based on tests/refserver.perl
 # Contributed by Artur Bergman <artur@vogon-solutions.com>
 # Revised for 0.06 by Rocco Caputo <troc@netrus.net>
 # Turned into a module by Philp Gwyn <fil@pied.nu>
 #
-# Copyright 1999,2001,2002,2004 Philip Gwyn.  All rights reserved.
+# Copyright 1999-2008 Philip Gwyn.  All rights reserved.
 # This program is free software; you can redistribute it and/or modify
 # it under the same terms as Perl itself.
 #
@@ -26,7 +26,7 @@ use Scalar::Util qw(reftype);
 require Exporter;
 @ISA = qw(Exporter);
 @EXPORT = qw(create_ikc_responder $ikc);
-$VERSION = '0.2000';
+$VERSION = '0.2001';
 
 sub DEBUG { 0 }
 
@@ -49,7 +49,8 @@ sub spawn
                       _start _stop
                       request post call raw_message post2
                       remote_error
-                      register unregister default register_local
+                      register unregister register_local register_channel
+                      default
                       publish retract subscribe unsubscribe
                       published
                       monitor inform_monitors shutdown 
@@ -133,6 +134,16 @@ sub default
     $heap->{self}->default($name);
 }
 
+#----------------------------------------------------
+# Register a channel.  So we can tell it to shutdown before it finishes
+# negociating
+sub register_channel
+{
+    my($heap, $channel, $rid, $aliases) = @_[HEAP, SENDER, ARG0, ARG1];
+    $heap->{self}->register_channel($channel);
+}
+
+
 
 ##############################################################################
 ## This state allows sessions to monitor a remote kernel
@@ -346,6 +357,7 @@ sub new
             rsvp=>{},
             kernel=>{},
             channel=>{},
+            channel_startup=>{},
             default=>{},
             monitors=>{},
             poe_kernel=>$kernel,
@@ -362,11 +374,23 @@ sub shutdown
         warn "$$: Some one wants us to go away... off we go\n";
     # kill our alias
     $kernel->alias_remove('IKC');
+
     # tell every channel to shutdown
     while(my($rid, $c)=each %{$self->{channel}}) {
-        DEBUG and warn "$$: Posting shutdown to $rid (id=$c)\n";
+        DEBUG and 
+                warn "$$: Posting shutdown to $rid (id=$c)\n";
         $kernel->post($c, 'shutdown');
     }
+    $self->{channel} = {};
+
+    # even the channels that haven't negociated yet
+    foreach my $c ( keys %{ $self->{channel_startup} } ) {
+        DEBUG and 
+                warn "$$: Posting shutdown to channel (id=$c)\n";
+        $kernel->post( $c, 'shutdown' );
+    }
+    $self->{channel_startup} = {};
+
     # tell monitors to shutdown
     $self->inform_monitors('*', 'shutdown');
     # kill pending subscription states
@@ -475,12 +499,16 @@ sub register
 
     my($kernel)=@{$self}{qw(poe_kernel)};
     
+    $channel = $channel->ID;
+    delete $self->{channel_startup}{ $channel };
+
     if($self->{channel}{$rid}) {
         warn "$$: Remote kernel '$rid' already exists\n";
         return;
     } 
     else {
-        DEBUG and warn "$$: Registered remote kernel '$rid'\n";
+        DEBUG and 
+            warn "$$: Registered remote kernel '$rid' (id=$channel)\n";
         $self->{channel}{$rid}=$channel;
         $self->{remote}{$rid}=[];       # list of proxy sessions
         $self->{alias}{$rid}=$aliases;  
@@ -545,6 +573,19 @@ sub register_local
     return 1;
 }
 
+#----------------------------------------------------
+# Register a starting channel
+sub register_channel
+{
+    my( $self, $channel ) = @_;
+    $channel = $channel->ID;
+    DEBUG and 
+        warn "$$: Registered channel (id=$channel)\n";
+    $self->{channel_startup}{ $channel } = 1;
+    return;
+}
+
+
 #----------------------------------------------------
 sub default
 {
@@ -572,6 +613,7 @@ sub unregister
     my($self, $channel, $rid, $aliases)=@_;
     my($kernel)=@{$self}{qw(poe_kernel)};
     return unless $rid;
+    $channel = $channel->ID;
 
     unless($aliases) {
         unless($self->{channel}{$rid}) {    # unregister one alias only
@@ -1973,6 +2015,15 @@ reasons.
 
 Philip Gwyn, <perl-ikc at pied.nu>
 
+=head1 COPYRIGHT AND LICENSE
+
+Copyright 1999-2008 by Philip Gwyn.  All rights reserved.
+
+This library is free software; you can redistribute it and/or modify
+it under the same terms as Perl itself.
+
+See L<http://www.perl.com/language/misc/Artistic.html>
+
 =head1 SEE ALSO
 
 L<POE>, 
